from onepassword_tools.lib.OnePasswordItem import OnePasswordItem


class OnePasswordDatabaseItem(OnePasswordItem):

    database: str = None
    hostname: str = None
    item_type: str = 'Database'
    notes: str = None
    password: str = None
    port: str = None
    url: str = None
    username: str = None

    def __getitem__(self, item):
        return super().__getitem__(item)

    def __init__(self, **kwargs):
        super().__init__()
        for key, value in kwargs.items():
            if hasattr(self, key):
                self.__setattr__(key, value)

    def _request_object(self):
        """
        Return the dict object with the request sent to 1Password
        :return: dict
        """
        return {
            "notesPlain": self.get('notes'),
            "sections": [
                {
                    "title": "",
                    "name": "Section_%s" % self.opu.generate_op_section_uuid(),
                    "fields": [
                        {
                            "k": "menu",
                            "n": self.opu.generate_op_field_uuid(),
                            "t": "type",
                        },
                        {
                            "k": "string",
                            "n": self.opu.generate_op_field_uuid(),
                            "v": self.hostname,
                            "t": "server"
                        },
                        {
                            "inputTraits": {
                                "keyboard": "NumberPad"
                            },
                            "k": "string",
                            "n": self.opu.generate_op_field_uuid(),
                            "v": self.port,
                            "t": "port"
                        },
                        {
                            "inputTraits": {
                                "autocapitalization": "none",
                                "autocorrection": "no"
                            },
                            "k": "string",
                            "n": self.opu.generate_op_field_uuid(),
                            "v": self.database,
                            "t": "database"
                        },
                        {
                            "k": "string",
                            "n": self.opu.generate_op_field_uuid(),
                            "v": self.username,
                            "t": "username"
                        },
                        {
                            "k": "concealed",
                            "n": self.opu.generate_op_field_uuid(),
                            "v": self.password,
                            "t": "password"
                        },
                        {
                            "k": "string",
                            "n": self.opu.generate_op_field_uuid(),
                            "t": "SID"
                        },
                        {
                            "k": "string",
                            "n": self.opu.generate_op_field_uuid(),
                            "t": "alias"
                        },
                        {
                            "k": "string",
                            "n": self.opu.generate_op_field_uuid(),
                            "t": "connection options"
                        }
                    ]
                }
            ]
        }



from onepassword_tools.lib.OnePasswordItem import OnePasswordItem


class OnePasswordLoginItem(OnePasswordItem):

    item_type: str = 'Login'
    notes: str = None
    password: str = None
    tags: [] = ['Compte utilisateur']
    url: str = None
    username: str = None

    def __getitem__(self, item):
        return super().__getitem__(item)

    def __init__(self, **kwargs):
        super().__init__()
        for key, value in kwargs.items():
            if hasattr(self, key):
                self.__setattr__(key, value)

    def _request_object(self):
        """
        Return the dict object with the request sent to 1Password
        :return: dict
        """
        return {
            "notesPlain": self.get('notes'),
            "sections": [],
            "fields": [
                {
                    "value": self.username,
                    "name": "username",
                    "type": "T",
                    "designation": "username"
                },
                {
                    "value": self.password,
                    "name": "password",
                    "type": "P",
                    "designation": "password"
                }
            ]
        }




import click


class ClickUtils:

    @staticmethod
    def error(message):
        click.echo(click.style(message, fg='red', bold=True))

    @staticmethod
    def info(message):
        click.echo(message)

    @staticmethod
    def success(message):
        click.echo(click.style(message, fg='green'))


from onepassword_tools.lib.ConfigFile import ConfigFile
import secrets
import string
from uuid import UUID


def generate_password(password_length):
    characters = string.ascii_letters + string.digits + r"""-_=.*+"""
    return ''.join(secrets.choice(characters) for i in range(password_length))


def is_uuid(uuid_string, version=4):
    try:
        uid = UUID(uuid_string, version=version)
        return uid.hex == uuid_string.replace('-', '')
    except ValueError:
        return False


def remove_null_value_keys_in_dict(data, only_keys=None):
    if only_keys is None:
        only_keys = ['v', 'value']
    new_data = {}
    for k, v in data.items():
        if isinstance(v, dict):
            v = remove_null_value_keys_in_dict(v)
        elif isinstance(v, list):
            clean_list = []
            for el in v:
                clean_list.append(remove_null_value_keys_in_dict(el))
            v = clean_list
        if k in only_keys and v in (u'', None, {}):
            continue
        new_data[k] = v
    return new_data


class SimpleFormatter(string.Formatter):

    def get_value(self, key, args, kwargs):
        return args[0].get(key)



import click
import copy
import json
import sys
from onepassword_tools.lib.OnePasswordUtils import OnePasswordUtils
from onepassword_local_search.OnePassword import OnePassword
from onepassword_tools.lib.MiscUtils import generate_password
from onepassword_tools.lib.MiscUtils import SimpleFormatter
from onepassword_tools.lib.OnePasswordResult import OnePasswordResult
from onepassword_tools.lib.ClickUtils import ClickUtils
from abc import ABCMeta, abstractmethod


def new_item_command_options(function):
    function = click.option('--notes', help='Note')(function)
    function = click.option('--title', help='Name of the 1Password item', default='')(function)
    function = click.option('--vault', help='Vault uuid where to store the information')(function)
    function = click.option('--account', help='Account to use (shorthand)')(function)
    function = click.option('--return-field', help='Field value to return', default=None)(function)
    function = click.option('--do-not-ask-credentials', help='Do not ask for credentials if not logged in', default=False, is_flag=True)(function)
    return function


def new_item_command_password(function):
    function = click.option('--password', help='Password to use, default autogenerated')(function)
    function = click.option('--password-length', help='Autogenerated password length, default 25', default=25, type=int)(function)
    return function


class NewItemCommand:
    __metaclass__ = ABCMeta
    account: str = None
    do_not_ask_credentials: bool = False
    notes: str = None
    onePassword: OnePassword
    onePasswordUtils: OnePasswordUtils
    password: str = None
    password_length: int = 25
    return_field: str = None
    title: str = None
    vault: str = None

    @property
    @abstractmethod
    def titleTemplate(self) -> str:
        pass

    @property
    @abstractmethod
    def onePasswordItemClass(self) -> str:
        pass

    def __init__(self, **kwargs):
        self.onePasswordUtils = OnePasswordUtils()
        self.onePassword = OnePassword()
        self._init(**kwargs)
        if self.password is None:
            self.password = generate_password(self.password_length)

    def _init(self, **kwargs):
        for key, value in kwargs.items():
            setattr(self, key, value)

    def authenticate_if_needed(self, account=None, check_mode=None):
        if not self.onePasswordUtils.is_authenticated(account=account, check_mode=check_mode):
            if self.do_not_ask_credentials:
                ClickUtils.error('You are not authenticated.')
                sys.exit(1)
            else:
                self.onePasswordUtils.authenticate(account)

    def get(self, attr):
        return getattr(self, attr)

    def save_on_1password(self, one_password_item_class):
        arguments = vars(copy.copy(self))
        del arguments['onePassword']
        del arguments['onePasswordUtils']
        server_item = one_password_item_class(**arguments)
        request_object = server_item.get_request_object()
        return self.onePasswordUtils.create_item(
            account=self.account,
            request_object=request_object,
            template=server_item.item_type,
            title=self.get_title(server_item),
            vault=self.vault
        )

    def get_title(self, item=None):
        template = self.titleTemplate if (self.title is None or self.title == '') else self.title
        if item is None:
            item = self
        sf = SimpleFormatter()
        return sf.format(template, item).strip()

    def run(self):

        self.authenticate_if_needed(account=self.account, check_mode='remote')

        item = self.save_on_1password(self.onePasswordItemClass)
        if 'uuid' in item.keys():
            result = ''
            if self.return_field is None:
                print(json.dumps(item))
            else:
                if self.return_field in item.keys():
                    result = item[self.return_field]
                else:
                    result = OnePasswordResult(dict(details=item['request_object'])).get(self.return_field)
            print(result)
            sys.exit(0)
        else:
            ClickUtils.error('Unable to save entry in 1Password')
            sys.exit(1)


import json
import logging
logger = logging.getLogger("app")


class Log:

    @staticmethod
    def _pretty_print(msg):
        if type(msg).__name__ == 'dict':
            msg = json.dumps(msg, indent=4)
        return msg

    @staticmethod
    def debug(msg, name='', pretty_print=True):
        if pretty_print:
            msg = Log._pretty_print(msg)
        if name:
            logger.debug('** %s **' % name)
        logger.debug(msg)
        logger.debug('')

    @staticmethod
    def debug2(msg, name='', pretty_print=True):
        if pretty_print:
            msg = Log._pretty_print(msg)
        if name:
            logger.log(5, '>> %s' % name)
        logger.log(5, msg)
        logger.log(5, '')

    @staticmethod
    def error(msg):
        logger.error(msg)

    @staticmethod
    def info(msg, name='', pretty_print=False):
        if pretty_print:
            msg = Log._pretty_print(msg)
        if name:
            logger.info('** %s **' % name)
        logger.info(msg)




import yaml
import os


class ConfigFile:

    path: str
    config: {} = {}

    def __init__(self):
        self.path = self.get_path()
        if os.path.isfile(self.path):
            with open(self.path, 'r') as ymlconfig:
                self.config = yaml.load(ymlconfig, yaml.SafeLoader)

    def config_key_exists(self, *keys):
        return self.keys_exists(self.config, *keys)

    @staticmethod
    def get_path():
        return os.path.expandvars(os.path.join('$HOME', '.onepassword-tools.yml'))

    def get_section(self, section):
        if section in self.config.keys():
            return self.config[section]
        return None

    @staticmethod
    def keys_exists(element, *keys):
        if type(element) is not dict:
            return False
        if len(keys) == 0:
            return False

        _element = element
        for key in keys:
            try:
                _element = _element[key]
            except KeyError:
                return False
        return True
